(def ks-version "0.7.2")
(def tk-version "0.5.1")

(defn deploy-info
  [url]
  {:url url
   :username :env/nexus_jenkins_username
   :password :env/nexus_jenkins_password
   :sign-releases false})

(defproject puppetlabs/pe-rbac-service "0.1.11"
  :description "Trapperkeeper service definitions for PE's RBAC system"
  :url "https://github.com/puppetlabs/pe-rbac-service"
  :license {:name ""
            :url ""}

  ;; TODO pedantic breaks with the inclusion of
  ;; pe-trapperkeeper-ldap-apacheds. Unless we can figure out how to
  ;; disable pedantic in our dev profile, it must be globally
  ;; disabled.
  ;:pedantic? :abort

  :repositories [["releases" "http://nexus.delivery.puppetlabs.net/content/repositories/releases/"]
                 ["snapshots"  "http://nexus.delivery.puppetlabs.net/content/repositories/snapshots/"]]

  :source-paths ["src/clj"]
  :java-source-paths ["src/java"]
  :dependencies [[org.clojure/clojure "1.6.0"]
                 ;; jdbc utils
                 [org.clojure/java.jdbc "0.3.2"]
                 ;[org.clojure/tools.logging "0.2.6"]
                 [cheshire  "5.3.1"]
                 [clj-stacktrace "0.2.7"]
                 ;; bonecp (database connection pool)
                 [com.jolbox/bonecp "0.8.0.RELEASE" :exclusions [org.slf4j/slf4j-api]]
                 [compojure "1.1.6" :exclusions [org.clojure/tools.macro
                                                 clj-time/clj-time]]
                 ;; for REST API
                 [liberator "0.11.0" :exclusions [org.clojure/tools.logging ]]
                 ;; migratus (database schema migration)
                 [migratus "0.7.0" :exclusions [org.clojure/tools.logging]]
                 ;; shiro (authentication/authorization)
                 [org.apache.shiro/shiro-core "1.2.2" :exclusions [org.slf4j/slf4j-api]]
                 [org.apache.shiro/shiro-web "1.2.2"]
                 ;; for syncing subject info
                 [org.clojars.pntblnk/clj-ldap "0.0.7"]
                 ;; postgres driver
                 [org.postgresql/postgresql "9.3-1100-jdbc41"]
                 ;; for parity with Classifier / data validation
                 [prismatic/schema "0.2.2"]
                 [puppetlabs/schema-tools "0.1.0" :exclusions [prismatic/schema]]
                 [puppetlabs/trapperkeeper ~tk-version]
                 [slingshot "0.10.3"]
                 [puppetlabs/clj-jwt "0.0.9-pl1" :exclusions [org.bouncycastle/bcprov-jdk15
                                                              clj-time/clj-time
                                                              org.clojure/data.json]]]

  :plugins [[migratus-lein "0.1.0" :exclusions [org.clojure/clojure]]
            [lein-release "1.0.5"]]

  :repl-options {:init (do
                         (require 'spyscope.core)
                         (use 'puppetlabs.rbac.testutils.repl
                              'puppetlabs.rbac.testutils.db
                              'puppetlabs.rbac.db-utils
                              'puppetlabs.rbac.testutils.fixtures))}

  ;; By declaring a classifier here and a corresponding profile below we'll get an additional jar
  ;; during `lein jar` that has all the code in the test/ directory. Downstream projects can then
  ;; depend on this test jar using a :classifier in their :dependencies to reuse the test utility
  ;; code that we have.
  :classifiers [["test" :testutils]]

  :profiles {:dev {:dependencies [[org.clojure/tools.namespace "0.2.4"]
                                  [clj-http "0.7.9"]
                                  [ring-mock "0.1.5"]
                                  [spyscope "0.1.4" :exclusions [clj-time]]
                                  [puppetlabs/certificate-authority "0.6.0"]
                                  [puppetlabs/http-client "0.2.6" :exclusions [commons-io]]
                                  [puppetlabs/kitchensink ~ks-version :classifier "test"]
                                  [puppetlabs/pe-trapperkeeper-ldap-apacheds "0.2.9"
                                   :exclusions [bouncycastle/bcprov-jdk15 org.slf4j/slf4j-log4j12 ]]
                                  [puppetlabs/trapperkeeper ~tk-version :classifier "test"]
                                  ;; Jetty 9 version of webserver-service requires Java 1.7
                                  [puppetlabs/trapperkeeper-webserver-jetty9 "0.7.2"]]
                   :injections    [(require 'spyscope.core)]}
             :testutils {:source-paths ^:replace ["test"]}}

  :test-selectors {:default (complement :integration)
                   :integration :integration
                   :unit (complement :integration)
                   :all (constantly true)}

  :aliases {"rbac" ["trampoline" "run" "--config" "dev-resources/conf.d"]
            "initdb" ["run" "--config" "dev-resources/conf.d" "--bootstrap-config" "resources/initdb.cfg"]}

  :migratus {:store :database
             :migration-dir "puppetlabs/rbac/migrations"
             :db {:classname "com.postgresql.jdbc.Driver"
                  :subprotocol "postgresql"
                  :subname "//localhost/perbac"
                  :user "perbac"
                  :password "foobar"}}

  :main puppetlabs.trapperkeeper.main

  :deploy-repositories [["releases" ~(deploy-info "http://nexus.delivery.puppetlabs.net/content/repositories/releases/")]
                        ["snapshots" ~(deploy-info "http://nexus.delivery.puppetlabs.net/content/repositories/snapshots/")]]
  :lein-release {:scm :git, :deploy-via :lein-deploy}
  :jvm-opts ["-XX:MaxPermSize=128M"])
