CREATE TABLE subject_digests (
  id           CHAR(40) NOT NULL PRIMARY KEY,  -- a hash code based on (subject_type, subject_id, subject_name)
  subject_type TEXT NOT NUll,
  subject_id   TEXT NOT NULL,
  subject_name TEXT NOT NULL,
  UNIQUE (subject_type, subject_id, subject_name)
);
--;;

CREATE INDEX ON subject_digests (subject_type, subject_id);
--;;

CREATE TABLE object_digests (
  id           CHAR(40) NOT NULL PRIMARY KEY,  -- a hash code based on (object_type, object_id, object_name)
  object_type  TEXT NOT NULL,
  object_id    TEXT NOT NULL,
  object_name  TEXT NOT NULL,
  UNIQUE (object_type, object_id, object_name)
);
--;;

CREATE INDEX ON object_digests (object_type, object_id);
--;;

CREATE TABLE event_commits (
  id                CHAR(40) NOT NULL PRIMARY KEY,  -- a hash code based on (service_id, subject_digest_id, object_digest_id, submit_time)
  service_id        TEXT NOT NULL,
  subject_digest_id TEXT NOT NULL,
  object_digest_id  TEXT NOT NULL,
  submit_time       TIMESTAMP WITH TIME ZONE NOT NULL,
  UNIQUE (service_id, subject_digest_id, object_digest_id, submit_time),
  FOREIGN KEY (subject_digest_id) REFERENCES subject_digests (id)
                          ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED,
  FOREIGN KEY (object_digest_id) REFERENCES object_digests (id)
                          ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
);
--;;

CREATE INDEX ON event_commits (service_id, subject_digest_id, submit_time);
--;;

CREATE INDEX ON event_commits (service_id, object_digest_id, submit_time);
--;;

CREATE INDEX event_commits_submit_time_idx ON event_commits (submit_time);
--;;

CREATE TABLE events (
  id                 CHAR(40) NOT NULL PRIMARY KEY,    -- a hash code based on (commit_id, action_type, action_what)
  commit_id          TEXT NOT NULL,
  action_type        TEXT NOT NULL,
  action_what        TEXT NOT NULL,
  action_description TEXT NOT NULL,
  message            TEXT NOT NULL,
  UNIQUE (commit_id, action_type, action_what, action_description),
  FOREIGN KEY (commit_id) REFERENCES event_commits (id)
                          ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
);
--;;

CREATE INDEX ON events (commit_id, message);
--;;

CREATE INDEX ON events (commit_id, action_type, action_what);
--;;
